'''MIKE!!!'''

MOVESPEED = 150.0   # pixels per second
FRAMETIME = 200.0   # milliseconds per frame
MAXTIME = 4000.0    # milliseconds per action
MINTIME =  500.0
BIRDDIST = 150.0    # how far away george tries to keep from bird, in px
MINBIRDY = 160.0    # min y before george can't see bird anymore
FALLSPEED = 800.0   # px per second
CAVEY = 550.0
GROUNDY = 350.0

import pygame
import random
import mob

# this should really be a subclass of Mob, but wrote this one first
class George(pygame.sprite.Sprite):
    def __init__(self, g):
        pygame.sprite.Sprite.__init__(self)
        self.g = g
        self.state = 'stand'
        self.basename = 'george'
        self.type = 'george'
        self.collidable = True
        self.direction = 0
        self.animstate = self.state
        self.animframe = 1
        self.animtimeremain = 0.0
        self.timeremain = 0.0
        self.dt = 0
        self.xpos = 400.0
        self.ypos = 350.0
        self.lastx = self.xpos
        self.lasty = self.ypos
        self._setimage()
        self.rect = self.image.get_rect()
        self.rect.midbottom = (self.xpos, self.ypos)
        self.bird = None
        self.falling = False
        self.touchelevator = False
        self.birdtop = MINBIRDY
        self.birdbottom = GROUNDY

    def _setimage(self):
        imgname = self.basename + '-' + self.animstate + '-' \
            + str(self.animframe)
        self.image = self.g.res.sprites[imgname][self.direction]
        self.rect = self.image.get_rect()
        self.rect.midbottom = (self.xpos, self.ypos)
        
    def reverse(self):
        self.direction = 1 - self.direction
        self._setimage()

    def _ai(self):
        if self.state == 'filmstand' or self.state == 'filmwalk':
            # do we still have bird?
            if self.bird.rect.bottom < self.birdbottom and \
                    self.bird.rect.bottom > self.birdtop:
                if abs(self.bird.rect.centerx - self.rect.centerx) > BIRDDIST:
                    self.state = 'filmwalk'
                else:
                    self.state = 'filmstand'
                if self.bird.rect.centerx < self.rect.centerx:
                    self.direction = 0
                else:
                    self.direction = 1
                self.timeremain = MINTIME
                return
            else:
                self.bird = None
                if self.state == 'filmstand':
                    self.state = 'stand'
                else:
                    self.state = 'walk'
            
        if self.state == 'stand':
            if random.random() > 0.5:
                self.state = 'walk'
                self.timeremain = random.random() * MAXTIME + MINTIME
        elif self.state == 'walk':
            if random.random() > 0.75:
                self.state = 'stand'
                self.timeremain = random.random() * MAXTIME + MINTIME
        if random.random() > 0.75:
            self.reverse()
        else:
            self._setimage()

    def _walk(self):
        newpos = self.xpos - (1 - self.direction * 2) * \
            (MOVESPEED * self.dt / 1000.0)
        if newpos < 75 or newpos > 725:
            newpos = self.xpos
            self.reverse()
        self.xpos = newpos
        self.rect.midbottom = (self.xpos, self.ypos)

    def _anim(self):
        if self.animstate != self.state:
            self.animstate = self.state
            self.animframe = 1
            self.animtimeremain = FRAMETIME
            self._setimage()
        self.animtimeremain -= self.dt
        if self.animtimeremain > 0:
            return
        self.animtimeremain = FRAMETIME
        if self.state == 'walk' or self.state == 'filmwalk':
            self.animframe = self.animframe % 2 + 1
        elif self.state == 'stand':
            pass
        self._setimage()

    def _find_bird(self):
        bird = None
        for sprite in self.g.scene.spritegroup.sprites():
            if sprite.type != 'bird' or sprite.state == 'wait':
                continue
            if sprite.rect.bottom < self.birdbottom and \
                    sprite.rect.bottom > self.birdtop:
                bird = sprite
        if bird == None:
            return
        self.state = 'filmstand'
        self.timeremain = MINTIME
        if bird.rect.centerx < self.rect.centerx:
            self.direction = 0
        else:
            self.direction = 1
        self.bird = bird
            
    def update(self, dt):
        if dt == 0:
            return
        self.dt = dt
        if self.ypos <= 400.0:
            self.birdtop = MINBIRDY
            self.birdbottom = GROUNDY
        else:
            self.birdtop = GROUNDY
            self.birdbottom = CAVEY
        if self.touchelevator == True:
            self.touchelevator = False
        elif self.falling == True:
            if self.ypos < CAVEY:
                self.ypos += FALLSPEED * dt / 1000.0
                self._setimage()
            else:
                self.falling = False
                self.ypos = CAVEY
                self.bird = None
        # Look for a bird
        if self.bird == None:
            self._find_bird()
        # See if bird we have is gone
        elif self.bird.state == 'wait':
            self.bird = None
            self.state = 'stand'
        # Only make a choice if time remaining on current state is zero
        if self.timeremain == 0:
            self._ai()
        # Peform actions for current state
        if self.state == 'walk' or self.state == 'filmwalk':
            self._walk()
        # Update animation
        self._anim()
        self.timeremain -= dt
        if self.timeremain < 0:
            self.timeremain = 0

    def shot(self):
        print "george got shot"

    def angry(self, amount = 1):
        self.g.scene.anger += amount
        self.g.snd.mike()

