'''All moveable objects'''

MOVESPEED = 150.0
FRAMETIME = 200.0
MAXTIME = 4000.0
MINTIME = 500.0
MAXX = 725.0
MINX = 75.0
MINY = 100.0
GROUNDY = 350.0
CAVEY = 550.0
FALLSPEED = 800.0

import pygame
from random import *

class Mob(pygame.sprite.Sprite):
    def __init__(self, g, basename, xpos = 400.0, ypos = 350.0):
        pygame.sprite.Sprite.__init__(self)
        self.g = g
        self.state = 'stand'
        self.basename = basename
        self.type = 'generic'
        self.collidable = False
        self.direction = 0
        self.falling = False
        self.touchelevator = False
        self.animstate = self.state
        self.animframe = 1
        self.animtimeremain = 0.0
        self.frametime = FRAMETIME
        self.timeremain = 0.0
        self.dt = 0
        self.xpos = xpos
        self.ypos = ypos
        self.lastx = self.xpos
        self.lasty = self.ypos
        self.xspeed = 0.0
        self.yspeed = 0.0
        self._setimage()
        self.rect = self.image.get_rect()
        self.rect.midbottom = (self.xpos, self.ypos)

    def _setimage(self):
        if self.animstate == 'runaway' or self.animstate == 'wait':
            if self.type == 'bird':
                anim = 'fly'
            else:
                anim = 'walk'
        else:
            anim = self.animstate
        imgname = self.basename + '-' + anim + '-' + str(self.animframe)
        self.image = self.g.res.sprites[imgname][self.direction]
        self.rect = self.image.get_rect()
        self.rect.midbottom = (self.xpos, self.ypos)

    def reverse(self):
        self.direction = 1 - self.direction
        self.xspeed = -self.xspeed
        self._setimage()

    def _ai(self):
        # Override this method to make default ai for each subclass
        pass

    def _animstate(self):
        # Override this method to make anim state transitions
        pass

    def _anim(self):
        if self.animstate != self.state:
            self.animstate = self.state
            self.animframe = 1
            self.animtimeremain = self.frametime
            self._setimage()
        self.animtimeremain -= self.dt
        if self.animtimeremain > 0:
            return
        self.animtimeremain = self.frametime
        self._animstate()

    def _process_state(self):
        # Override this method to make main state transitions
        pass

    def update(self, dt):
        if dt == 0:
            return
        self.dt = dt
        # Only make a choice if time remaining on current state is zero
        if self.timeremain == 0:
            self._ai()
        # Peform actions for current state
        self._process_state()
        # Update animation
        self._anim()
        self.timeremain -= dt
        if self.timeremain < 0:
            self.timeremain = 0

    def _runaway(self):
        self.state = 'runaway'
        if random() > 0.5:
            self.xspeed = -2 * MOVESPEED
            self.direction = 0
        else:
            self.xspeed = 2 * MOVESPEED
            self.direction = 1
        self.frametime = FRAMETIME / 2
        self.timeremain = 0

    def shot(self):
        self._runaway()

    def hitgeorge(self):
        pass
