'''target practice scratch scene'''
import scene
import george
import bird
import critter
import pygame
import hud
import hole
import elevator
import trigger
from pygame.locals import *

class TargetPractice(scene.Scene):
    def __init__(self, g):
        self.g = g
        self.state = 'init'
        self.bgimage = g.res.images['trees-bg-1']
        self.score = 0.0
        self.anger = 0
        if g.level == 1:
            self.timethislevel = 1000.0 * 60.0 * 0.5
        else:
            self.timethislevel = 1000.0 * 60.0 * 1.0
        self.timeelapsed = 0.0
        self.george = george.George(g)
        self.spritegroup = pygame.sprite.OrderedUpdates()
        self.loadlevel(g.level)
        self.hud = hud.Hud(g)
        self.g.snd.playmusic()
        # Render the entire screen once since further updates are per rect
        self.g.screen.blit(self.bgimage, (0, 0))
        pygame.display.flip()

    def simstep(self, dt):
        self.userinput(dt)
        self.collisions(dt)
        self.spritegroup.clear(self.g.screen, self.bgimage)
        self.spritegroup.update(dt)
        self.hud.update(dt)
        self.timeelapsed += dt
        if self.anger > 5:
            self.g.switch_to_scene = 'gameover-lose'
        if self.timeelapsed > self.timethislevel:
            self.g.level += 1
            if self.g.level > 2:
                self.g.switch_to_scene = 'gameover-win'
            else:
                self.g.switch_to_scene = 'levelcomplete'
                self.spritegroup.empty()
            #    self.loadlevel(self.g.level)
                self.timethislevel = 1000.0 * 60.0 * 2
                self.selftimeelapsed = 0.0

    def display(self):
        rectlist = self.spritegroup.draw(self.g.screen)
        pygame.display.update(rectlist)

    def userinput(self, dt):
        for event in pygame.event.get():
            if event.type == KEYDOWN:
                if event.key == K_ESCAPE or event.key == K_q:
                    self.g.run = False
                    return
            if event.type == MOUSEBUTTONDOWN:
                for sprite in self.spritegroup:
                    if sprite.rect.collidepoint(event.pos):
                        sprite.shot()
                        self.g.snd.bing()

    def collisions(self, dt):
        for sprite in pygame.sprite.spritecollide(self.george, 
                                                  self.spritegroup, False):
            if sprite.collidable == False or sprite == self.george:
                continue
            sprite.hitgeorge()
        # holes
        for hole in self.spritegroup:
            if hole.type != 'hole':
                continue
            for sprite in pygame.sprite.spritecollide(hole, self.spritegroup,
                                                      False):
                if sprite.collidable == False:
                    continue
                if sprite.rect.centerx < hole.rect.right and \
                        sprite.rect.centerx > hole.rect.left:
                    sprite.falling = True
        # elevators
        for elevator in self.spritegroup:
            if elevator.type != 'elevator':
                continue
            for sprite in pygame.sprite.spritecollide(elevator, 
                                                      self.spritegroup, False):
                if sprite.collidable == False:
                    continue
                if sprite.rect.centerx < elevator.rect.right and \
                        sprite.rect.centerx > elevator.rect.left:
                    sprite.touchelevator = True
                    if elevator.state == 'top':
                        sprite.falling = False
                    sprite.ypos = elevator.rect.centery
                    sprite._setimage()

    def loadlevel(self, level):
        if level == 1:
            self.spritegroup.add(hole.Hole(self.g, 100.0))
            lift = elevator.Elevator(self.g, 100.0)
            self.spritegroup.add(lift)
            self.spritegroup.add(trigger.Trigger(self.g, lift, 170.0, 585.0))
            self.spritegroup.add(bird.Bird(self.g, xpos = 200.0, ypos = 200.0))
            self.spritegroup.add(critter.Critter(self.g, xpos = 700.0))
        elif level == 2:
            self.spritegroup.add(hole.Hole(self.g, 100.0))
            self.spritegroup.add(hole.Hole(self.g, 700.0))
            lift = elevator.Elevator(self.g, 100.0)
            self.spritegroup.add(lift)
            self.spritegroup.add(trigger.Trigger(self.g, lift, 170.0, 585.0))
            lift = elevator.Elevator(self.g, 700.0)
            self.spritegroup.add(lift)
            self.spritegroup.add(trigger.Trigger(self.g, lift, 770.0, 585.0))
            self.spritegroup.add(bird.Bird(self.g, 'bat', 400.0, 450.0))
            self.spritegroup.add(critter.Critter(self.g, 'skunk', 200.0))
            self.spritegroup.add(critter.Critter(self.g, 'skunk', 600.0))
            self.spritegroup.add(critter.Critter(self.g, 'skunk', 400.0, 
                                                 550.0))

        self.spritegroup.add(self.george)
